/*--------------------------------------------------------------------------------------+
|
|     $Source: MstnExamples/View/DisplayRule/DisplayRuleCreateTool.cpp $
|
|  $Copyright: (c) 2021 LA Solutions Ltd. All rights reserved. $
|  $Copyright: (c) 2015 Bentley Systems, Incorporated. All rights reserved. $
|
+--------------------------------------------------------------------------------------*/
#include "displayrule.h"


USING_NAMESPACE_BENTLEY_DGNPLATFORM;
USING_NAMESPACE_BENTLEY_MSTNPLATFORM;
USING_NAMESPACE_BENTLEY_MSTNPLATFORM_ELEMENT;

/*=================================================================================**//**
* Example showing how to create a display rule.
*
* @bsiclass
+===============+===============+===============+===============+===============+======*/
struct          DisplayRuleCreateTool : DgnPrimitiveTool
{
protected:

bvector<DPoint3d>   m_points;

DisplayRuleCreateTool(int toolName, int toolPrompt) : DgnPrimitiveTool(toolName, toolPrompt) {}

virtual void _OnPostInstall () override;
virtual void _OnRestartTool () override {InstallNewInstance (GetToolId (), GetToolPrompt ());}
virtual bool _OnDataButton (DgnButtonEventCR ev) override;
virtual bool _OnResetButton (DgnButtonEventCR ev) override {_OnRestartTool (); return true;}

int            GetIndexForDisplayStyle(WCharCP name, DgnFileR dgnFile);
void        AddDifferentDisplayRuleSetsToFile(DgnFileR dgnFile);

public:

static void InstallNewInstance (int toolId, int toolPrompt);

}; // DisplayRuleCreateTool

/*---------------------------------------------------------------------------------**//**
* @bsimethod                                                              Bentley Systems
+---------------+---------------+---------------+---------------+---------------+------*/
int DisplayRuleCreateTool::GetIndexForDisplayStyle(WCharCP name, DgnFileR dgnFile)
    {
    DisplayStyleCP      style;
    for (int index = 0; NULL != (style = DisplayStyleManager::GetDisplayStyleByIndex(index, dgnFile)); index++)
        if (style->GetName().Equals(name))
            return index;

    return -1;
    }
/*---------------------------------------------------------------------------------**//**
* @bsimethod                                                              Bentley Systems
+---------------+---------------+---------------+---------------+---------------+------*/
void DisplayRuleCreateTool::AddDifferentDisplayRuleSetsToFile(DgnFileR dgnFile)
    {
        //1. Display style override rule set.
        {
        DisplayRuleSetPtr displayStyleRuleSet = DisplayRuleSet::Create(L"Test Display Style Overrides", dgnFile);

        int     styleIndex;

        if ((styleIndex = GetIndexForDisplayStyle(L"Monochrome", dgnFile)) >= 0)
            {
            DisplayRulePtr      displayStyleRule = DisplayRule::Create(L"element.DgnElementSchema::GraphicalElement::Color=1", true, dgnFile);

            displayStyleRule->GetActions().push_back(DisplayStyleOverrideAction::Create(styleIndex, dgnFile));
            displayStyleRuleSet->GetDisplayRules().push_back(displayStyleRule);
            }

        if ((styleIndex = GetIndexForDisplayStyle(L"Thematic:Height", dgnFile)) >= 0)
            {
            DisplayRulePtr      displayStyleRule = DisplayRule::Create(L"element.DgnElementSchema::GraphicalElement::Color=2", true, dgnFile);

            displayStyleRule->GetActions().push_back(DisplayStyleOverrideAction::Create(styleIndex, dgnFile));
            displayStyleRuleSet->GetDisplayRules().push_back(displayStyleRule);
            }
        if (displayStyleRuleSet->GetDisplayRules().size() > 0)
            DisplayRulesManager::WriteDisplayRuleSetToFile(*displayStyleRuleSet, dgnFile, false);
        }

        //2. Hatching Rule Set-Test Area Hatching
        {
        DisplayRuleSetPtr hatchRuleSet = DisplayRuleSet::Create(L"Test Area Hatching", dgnFile);

        Int32       style = 2;
        UInt32      weight = 2;

        DisplayRulePtr hatchRule2 = DisplayRule::Create(L"element.DgnElementSchema::GraphicalElement::Color=2", true, dgnFile);
        hatchRule2->GetActions().push_back(AreaHatchAction::Create(AreaHatchAction::HatchParams(.5, -msGeomConst_piOver4, NULL, NULL, false, NULL, &weight, &style), dgnFile));
        hatchRuleSet->GetDisplayRules().push_back(hatchRule2);

        double      angle2 = -msGeomConst_piOver4;
        double      distance2 = 1;
        UInt32      color = 0;

        DisplayRulePtr hatchRule3 = DisplayRule::Create(L"element.DgnElementSchema::GraphicalElement::Color=3", true, dgnFile);
        hatchRule3->GetActions().push_back(AreaHatchAction::Create(AreaHatchAction::HatchParams(.5, msGeomConst_piOver4, &distance2, &angle2, false, &color), dgnFile));
        hatchRuleSet->GetDisplayRules().push_back(hatchRule3);

        if (hatchRuleSet->GetDisplayRules().size() > 0)
            DisplayRulesManager::WriteDisplayRuleSetToFile(*hatchRuleSet, dgnFile, false);
        }

        {
        //3. Area Patterning Rule Set.
        ElementRefP     patternSharedCell = NULL;
        DisplayRuleSetPtr patternRuleSet = DisplayRuleSet::Create(L"Test Area Patterning", dgnFile);

        if (NULL != (patternSharedCell = ISharedCellQuery::FindDefinitionByName(L"BRICK", dgnFile)))
            {
            DisplayRulePtr patternRule1 = DisplayRule::Create(L"element.DgnElementSchema::GraphicalElement::Color=1", true, dgnFile);
            patternRule1->GetActions().push_back(AreaPatternAction::Create(AreaPatternAction::AreaPatternParams(patternSharedCell->GetElementId(), 1.0), dgnFile));
            patternRuleSet->GetDisplayRules().push_back(patternRule1);
            }

        if (NULL != (patternSharedCell = ISharedCellQuery::FindDefinitionByName(L"GRASS", dgnFile)))
            {
            DisplayRulePtr displayRule = DisplayRule::Create(L"element.DgnElementSchema::GraphicalElement::Color=2", true, dgnFile);
            displayRule->GetActions().push_back(AreaPatternAction::Create(AreaPatternAction::AreaPatternParams(patternSharedCell->GetElementId(), 1.0), dgnFile));
            patternRuleSet->GetDisplayRules().push_back(displayRule);
            }

        if (NULL != (patternSharedCell = ISharedCellQuery::FindDefinitionByName(L"CRSRBL", dgnFile)))
            {
            UInt32      color = 6;

            DisplayRulePtr displayRule = DisplayRule::Create(L"element.DgnElementSchema::GraphicalElement::Color=3", true, dgnFile);
            displayRule->GetActions().push_back(AreaPatternAction::Create(AreaPatternAction::AreaPatternParams(patternSharedCell->GetElementId(), 1.0, false, NULL, &color), dgnFile));
            patternRuleSet->GetDisplayRules().push_back(displayRule);
            }
        if (patternRuleSet->GetDisplayRules().size() > 0)
            DisplayRulesManager::WriteDisplayRuleSetToFile(*patternRuleSet, dgnFile, false);
        }

        //4. Hide an Element Display Rule set
        {
        DisplayRuleSetPtr elementDisplayRuleSet = DisplayRuleSet::Create(L"Test Element Display", dgnFile);

        DisplayRulePtr      displayStyleRule = DisplayRule::Create(L"element.DgnElementSchema::GraphicalElement::Color=1", true, dgnFile);
        displayStyleRule->GetActions().push_back(ElementDisplayAction::Create(false));
        elementDisplayRuleSet->GetDisplayRules().push_back(displayStyleRule);
        if (elementDisplayRuleSet->GetDisplayRules().size() > 0)
            DisplayRulesManager::WriteDisplayRuleSetToFile(*elementDisplayRuleSet, dgnFile, false);
        }

        //5. Changes Element Color Rule set
        {
        DisplayRuleSetPtr colorOverrideDisplayRuleSet = DisplayRuleSet::Create(L"Test Color Override", dgnFile);
        UInt32 colorOverride = 100;
        DisplayRulePtr      displayStyleRule = DisplayRule::Create(L"element.DgnElementSchema::GraphicalElement::Color=1", true, dgnFile);
        displayStyleRule->GetActions().push_back(ColorOverrideAction::Create(colorOverride,dgnFile));
        colorOverrideDisplayRuleSet->GetDisplayRules().push_back(displayStyleRule);
        if (colorOverrideDisplayRuleSet->GetDisplayRules().size() > 0)
            DisplayRulesManager::WriteDisplayRuleSetToFile(*colorOverrideDisplayRuleSet, dgnFile, false);
        }

        //6. Change Element fill color Rule set
        {
        DisplayRuleSetPtr fillColorOverrideDisplayRuleSet = DisplayRuleSet::Create(L"Test Fill Color Override", dgnFile);
        UInt32 fillColorOverride = 10;
        DisplayRulePtr      displayStyleRule = DisplayRule::Create(L"element.DgnElementSchema::GraphicalElement::Color=1", true, dgnFile);
        displayStyleRule->GetActions().push_back(FillColorOverrideAction::Create(fillColorOverride, dgnFile));
        fillColorOverrideDisplayRuleSet->GetDisplayRules().push_back(displayStyleRule);
        if (fillColorOverrideDisplayRuleSet->GetDisplayRules().size() > 0)
            DisplayRulesManager::WriteDisplayRuleSetToFile(*fillColorOverrideDisplayRuleSet, dgnFile, false);
        }

        //7. Change Element line style Rule set
        {
        DisplayRuleSetPtr lstyleOverrideDisplayRuleSet = DisplayRuleSet::Create(L"Test Line Style Override", dgnFile);
        Int32 lStyleOverride = 5;
        DisplayRulePtr      displayStyleRule = DisplayRule::Create(L"element.DgnElementSchema::GraphicalElement::Color=1", true, dgnFile);
        displayStyleRule->GetActions().push_back(StyleOverrideAction::Create(lStyleOverride, dgnFile));
        lstyleOverrideDisplayRuleSet->GetDisplayRules().push_back(displayStyleRule);
        if (lstyleOverrideDisplayRuleSet->GetDisplayRules().size() > 0)
            DisplayRulesManager::WriteDisplayRuleSetToFile(*lstyleOverrideDisplayRuleSet, dgnFile, false);
        }

        //8. Change Element Transparency Rule set
        {
        DisplayRuleSetPtr transparencyOverrideDisplayRuleSet = DisplayRuleSet::Create(L"Test Transparency Override", dgnFile);
        double transparencyOverride = 0.8;
        DisplayRulePtr      displayStyleRule = DisplayRule::Create(L"element.DgnElementSchema::GraphicalElement::Color=1", true, dgnFile);
        displayStyleRule->GetActions().push_back(TransparencyOverrideAction::Create(transparencyOverride));
        transparencyOverrideDisplayRuleSet->GetDisplayRules().push_back(displayStyleRule);
        if (transparencyOverrideDisplayRuleSet->GetDisplayRules().size() > 0)
            DisplayRulesManager::WriteDisplayRuleSetToFile(*transparencyOverrideDisplayRuleSet, dgnFile, false);
        }

        //9. Change Element Priority Rule set
        {
        DisplayRuleSetPtr priorityOverrideDisplayRuleSet = DisplayRuleSet::Create(L"Test Priority Override", dgnFile);
        int  elementPriority = 1;
        DisplayRulePtr      displayStyleRule = DisplayRule::Create(L"element.DgnElementSchema::GraphicalElement::Color=1", true, dgnFile);
        displayStyleRule->GetActions().push_back(ElementPriorityAction::Create(elementPriority));
        priorityOverrideDisplayRuleSet->GetDisplayRules().push_back(displayStyleRule);
        if (priorityOverrideDisplayRuleSet->GetDisplayRules().size() > 0)
            DisplayRulesManager::WriteDisplayRuleSetToFile(*priorityOverrideDisplayRuleSet, dgnFile, false);
        }
    }
/*---------------------------------------------------------------------------------**//**
* @bsimethod                                                              Bentley Systems
+---------------+---------------+---------------+---------------+---------------+------*/
bool DisplayRuleCreateTool::_OnDataButton(DgnButtonEventCR ev)
    {
    DgnFileP dgnFile = ev.GetViewport()->GetRootModel()->GetDgnFileP();
    AddDifferentDisplayRuleSetsToFile(*dgnFile);

    return true;
    }

/*---------------------------------------------------------------------------------**//**
* @bsimethod                                                              Bentley Systems
+---------------+---------------+---------------+---------------+---------------+------*/
void DisplayRuleCreateTool::_OnPostInstall()
    {
    AccuSnap::GetInstance ().EnableSnap (true); // Enable snapping for create tools.

    __super::_OnPostInstall ();
    }

/*---------------------------------------------------------------------------------**//**
* Method to create and install a new instance of the tool. If InstallTool returns ERROR,
* the new tool instance will be freed/invalid. Never call delete on RefCounted classes.
*
* @bsimethod                                                              Bentley Systems
+---------------+---------------+---------------+---------------+---------------+------*/
void DisplayRuleCreateTool::InstallNewInstance(int toolId, int toolPrompt)
    {
    DisplayRuleCreateTool* tool = new DisplayRuleCreateTool(toolId, toolPrompt);

    tool->InstallTool ();
    }

/*---------------------------------------------------------------------------------**//**
* Function that was associated with the command number for starting the tool.
*
* @param[in] unparsed Additional input supplied after command string.
* @bsimethod                                                              Bentley Systems
+---------------+---------------+---------------+---------------+---------------+------*/
Public void startDisplayRuleCreateTool(WCharCP unparsed)
    {
    // NOTE: Call the method to create/install the tool, RefCounted classes don't have public constructors...
    DisplayRuleCreateTool::InstallNewInstance(CMDNAME_DisplayRuleCreateTool, PROMPT_InputAPoint);
    }



