/*--------------------------------------------------------------------------------------+
|
|     $Source: MstnExamples/View/DisplayRule/DisplayRuleDeleteTool.cpp $
|
|  $Copyright: (c) 2015 Bentley Systems, Incorporated. All rights reserved. $
|
+--------------------------------------------------------------------------------------*/
#include "displayrule.h"


USING_NAMESPACE_BENTLEY_DGNPLATFORM;
USING_NAMESPACE_BENTLEY_MSTNPLATFORM;
USING_NAMESPACE_BENTLEY_MSTNPLATFORM_ELEMENT;

/*=================================================================================**//**
* Example showing how to delete a display rule from dgn file.
* @bsiclass
+===============+===============+===============+===============+===============+======*/
struct          DisplayRuleDeleteTool : DgnPrimitiveTool
{
protected:

bvector<DPoint3d>   m_points;

DisplayRuleDeleteTool(int toolName, int toolPrompt) : DgnPrimitiveTool(toolName, toolPrompt) {}

virtual void _OnRestartTool () override {InstallNewInstance (GetToolId (), GetToolPrompt ());}
virtual bool _OnDataButton (DgnButtonEventCR ev) override;
virtual bool _OnResetButton (DgnButtonEventCR ev) override {_OnRestartTool (); return true;}

void        DeleteDisplayRuleSetsFromFile(DgnFileR dgnFile);

public:

static void InstallNewInstance (int toolId, int toolPrompt);

}; // DisplayRuleDeleteTool


/*---------------------------------------------------------------------------------**//**
* @bsimethod                                                              Bentley Systems
+---------------+---------------+---------------+---------------+---------------+------*/
void DisplayRuleDeleteTool::DeleteDisplayRuleSetsFromFile(DgnFileR dgnFile)
    {
    DisplayRulesManager::RemoveDisplayRuleSetFromFile(L"Test Display Style Overrides", dgnFile);
    DisplayRulesManager::RemoveDisplayRuleSetFromFile(L"Test Area Hatching", dgnFile);
    DisplayRulesManager::RemoveDisplayRuleSetFromFile(L"Test Area Patterning", dgnFile);
    DisplayRulesManager::RemoveDisplayRuleSetFromFile(L"Test Element Display", dgnFile);
    DisplayRulesManager::RemoveDisplayRuleSetFromFile(L"Test Color Override", dgnFile);
    DisplayRulesManager::RemoveDisplayRuleSetFromFile(L"Test Fill Color Override", dgnFile);
    DisplayRulesManager::RemoveDisplayRuleSetFromFile(L"Test Line Style Override", dgnFile);
    DisplayRulesManager::RemoveDisplayRuleSetFromFile(L"Test Transparency Override", dgnFile);
    DisplayRulesManager::RemoveDisplayRuleSetFromFile(L"Test Priority Override", dgnFile);
    }
/*---------------------------------------------------------------------------------**//**
* @bsimethod                                                              Bentley Systems
+---------------+---------------+---------------+---------------+---------------+------*/
    bool DisplayRuleDeleteTool::_OnDataButton(DgnButtonEventCR ev)
    {
    DgnFileP dgnFile = ev.GetViewport()->GetRootModel()->GetDgnFileP();
    DeleteDisplayRuleSetsFromFile(*dgnFile);
    
    return true;
    }

/*---------------------------------------------------------------------------------**//**
* Method to create and install a new instance of the tool. If InstallTool returns ERROR,
* the new tool instance will be freed/invalid. Never call delete on RefCounted classes.
*
* @bsimethod                                                              Bentley Systems
+---------------+---------------+---------------+---------------+---------------+------*/
void DisplayRuleDeleteTool::InstallNewInstance(int toolId, int toolPrompt)
    {
    DisplayRuleDeleteTool* tool = new DisplayRuleDeleteTool(toolId, toolPrompt);

    tool->InstallTool ();
    }

/*---------------------------------------------------------------------------------**//**
* Function that was associated with the command number for starting the tool.
*
* @param[in] unparsed Additional input supplied after command string.
* @bsimethod                                                              Bentley Systems
+---------------+---------------+---------------+---------------+---------------+------*/
Public void startDisplayRuleDeleteTool(WCharCP unparsed)
    {
    // NOTE: Call the method to create/install the tool, RefCounted classes don't have public constructors...
    DisplayRuleDeleteTool::InstallNewInstance(CMDNAME_DisplayRuleDeleteTool, PROMPT_DeleteRules);
    }



